#ifndef __COMMONDEFS_H__
#define __COMMONDEFS_H__

// game configuration
#define MAX_STAGE (2) // # of maps available
#define VEL (16) // velocity of cars
#define MAX_SMOKE (24)
#define MAX_TRAIL (4)
#define MAX_FUEL (3000)
#define ALMOST_EMPTY_FUEL (600)
#define SMOKE_FUEL (150)
#define INIT_NLIFE (2)
#define INIT_TIMER_PERIOD (40)

// field map attributes
#define MAP_WIDTH (32)  // field map range
#define MAP_HEIGHT (56)
#define MAP_FOV (5)     // field of view on field map

// play field attributes
#define RES (64) // resolution of a single sprite
#define PF_WIDTH (MAP_WIDTH * RES)   // play field range
#define PF_HEIGHT (MAP_HEIGHT * RES) 
#define PF_FOV (RES * MAP_FOV)       // field of view on play field

// system constatns
#include "version.h"
#define PORT ((in_port_t)5555)
#define MAX_CLIENT (8)
#define MAX_NAMELEN (16)
#define MAX_DATA_SIZE (MAP_WIDTH * MAP_HEIGHT + 128)

// keycode <-> keyboard mapping
#define  MASK_UPKEY     (1<<0)
#define  MASK_DOWNKEY   (1<<1)
#define  MASK_LEFTKEY   (1<<2)
#define  MASK_RIGHTKEY  (1<<3)
#define  MASK_SPACEKEY  (1<<4)
#define  MASK_QKEY      (1<<5)
#define  MASK_SKEY      (1<<6)

// macros for coordinate system conversion.
#define M2PF(X, Y) (RES * (X)), (RES * (Y))
#define PF2GL(X, Y) (((X) - Me->x)/(double)PF_FOV), ((Me->y - (Y))/(double)PF_FOV)
#define M2GL(X, Y)  ((double)(RES * (X+0.5) - Me->x)/(double)PF_FOV), ((double)(Me->y - RES * (Y+0.5))/(double)PF_FOV)
#define M2GL_SCALE(X) ((X) * RES * 0.5 / (double)PF_FOV)
#define PF2M_SCALE(X) ((X) / (double)RES)
#define M2PF_SCALE(X) (RES * (X) + RES / 2)
#define WM2GL(X, Y)  ((double)(RES * (X+0.5) - Me->x)/(double)(RES*MAP_WIDTH)), ((double)(Me->y - RES * (Y+0.5))/(double)(RES*MAP_HEIGHT))

enum { // char data on map
    MAP_NONE,
    MAP_WALL,
    MAP_FLAG,
    MAP_LUCKY,
    MAP_SPECIAL,
    MAP_ROCK,
};

typedef enum {
    GAME_TITLE,
    GAME_READY,
    GAME_ONPLAY,
    GAME_FINISH,
    GAME_RESULT,
    GAME_FINALRESULT,
    GAME_RETIRE,
    GAME_OVER,
    GAME_QUIT,
    GAME_PAUSE,
    NGAME,
} Gamestatus;

// header tag of transfered packets
typedef enum {
    TAG_KEYPRESSED, // followed by a 4-byte encoded keyinput
    TAG_PLAYERINFO, // followed by essential info of all players
    TAG_PLAYERINFO2, // followed by verbose info of all players
    TAG_STAGEINFO, // followed by info of current stage
    TAG_MAP, // followed by MAP_HEIGHT * MAP_WIDTH byte data
    TAG_SMOKE, // followed by MAX_SMOKE smoke info (position & lifetime)
    TAG_HIGHSCORE, // followed highscore
    TAG_GAMETITLE,
    TAG_GAMEREADY,
    TAG_GAMESTART,
    TAG_GAMEFINISH,
    TAG_GAMERESULT,
    TAG_GAMEFINALRESULT,
    TAG_GAMERETIRE,
    TAG_GAMEOVER,
    TAG_GAMEQUIT,
    TAG_GAMEPAUSE,
    NTAG, // number of tags
} Packettag;
#define TAG_SIZE (sizeof(Packettag) + sizeof(int))
// packet size is followed by a TAG

typedef struct {
    int x, y; // position
    int dir;  // direction
    int status; // see enum {P_XXX}
    int fuel;
    // END OF ESSENTIAL DATA
    int nflag; // # of flags got
    int nlife; // # of spare cars
    int nwin; // # of stage wins
    int score; // game score
    int ntotalflag; // # of flags got whole through a game
    int ctrlstyle; // control algorithm of player move
    char name[MAX_NAMELEN];
    // END OF VERBOSE DATA

    // members below are used only on the server side
    int cnt; // counter used in P_CRASH & P_SMOKE status
    int flagscore; // score of next flag
    int special; // score doubling due to special flag (x1 or x2)
    int initx, inity; // position at beginning of a stage
    int vx, vy; // velocity
    int ax, ay; // acceleration
    unsigned int key; // keyboard input
    int trail[4][2]; // position of the latest four trails (map coordinate)
} Player;
#define POFF (int)(&((Player *)(NULL))->nflag) // size of essential data sent from server to clients
#define POFF2 (int)(&((Player *)(NULL))->cnt) // size of verbose data sent from server to clients

enum {
    P_NOTLOGGEDIN,
    P_NORMAL,
    P_CRASH,
    P_SPIN,
    P_FUELEMPTY,
    P_FINISH,
    P_RETIRE,
    P_CLASSIC,
    P_MODERN,
};

#endif // __COMMONDEFS_H__
