// wrapper for socket library routines

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>

#ifndef WIN32
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#endif
#include "utils.h"

int
setup_server(in_port_t port)
{
    struct sockaddr_in me;
    SOCKET soc_waiting;
    SOCKET soc;

    memset((char *)&me, 0, sizeof(me));
    me.sin_family = AF_INET;
    me.sin_addr.s_addr = htonl(INADDR_ANY);
    me.sin_port = htons(port);

#if defined(WIN32)
	{
		WSADATA wsaData;
	    WSAStartup(MAKEWORD(2,1), &wsaData);
	}
#endif

    soc_waiting = socket(AF_INET, SOCK_STREAM, 0);
    if (soc_waiting < 0) {
	perror("socket");
	return -1;
    }

    if (bind(soc_waiting, (struct sockaddr *)&me, sizeof(me)) == -1) {
	perror("bind");
	return -1;
    }

    listen(soc_waiting, 1);
    fprintf(stderr, "successfully bound. waiting for connection request.\n");

    soc = accept(soc_waiting, NULL, NULL);
    closesocket(soc_waiting);

    return soc;
}

int
mserver_socket(in_port_t port, int backlog)
{
    struct sockaddr_in me;
    SOCKET soc_waiting;

    memset((char *)&me, 0, sizeof(me));
    me.sin_family = AF_INET;
    me.sin_addr.s_addr = htonl(INADDR_ANY);
    me.sin_port = htons(port);

#if defined(WIN32)
	{
	    WSADATA wsaData;
	    WSAStartup(MAKEWORD(2,1), &wsaData);
	}
#endif

    soc_waiting = socket(AF_INET, SOCK_STREAM, 0);
    if (soc_waiting < 0) {
	perror("socket");
	return -1;
    }
	
    if (bind(soc_waiting, (struct sockaddr *)&me, sizeof(me)) == -1) {
#if defined(WIN32)
		fprintf(stderr, "bind failed");
#else
		perror("bind");
#endif
		return -1;
    }

    if (listen(soc_waiting, backlog) == -1) {
#if defined(WIN32)
		fprintf(stderr, "listen failed");
#else
		perror("listen");
#endif
		return -1;
    }
    fprintf(stderr, "successfully setup.\n");

    return soc_waiting;
}

int
mserver_maccept(SOCKET soc_waiting, int limit, void (*func)())
{
    int i;
    SOCKET fd;

    for (i = 0; i < limit; i++) {
		fd = accept(soc_waiting, NULL, NULL);
		if (fd == -1) {
			fprintf(stderr, "accept error\n");
			return -1;
		}
		(*func)(i, fd);
    }
    closesocket(soc_waiting);
    return fd;
}

int
setup_client(char *hostname, in_port_t port)
{
    struct hostent *server_ent;
    struct sockaddr_in server;
    SOCKET soc;

#if defined(WIN32)
	{
		WSADATA wsaData;
	    WSAStartup(MAKEWORD(2,1), &wsaData);
	}
#endif

    server_ent = gethostbyname(hostname);
    if (server_ent == NULL) {
#if defined(WIN32)
	fprintf(stderr, "gethostbyname failed");
#else
	perror("gethostbyname");
#endif
	return -1;
    }

    memset((char *)&server, 0, sizeof(server));
    server.sin_family = AF_INET;
    server.sin_port = htons(port);
    memcpy((char *)&server.sin_addr, server_ent->h_addr,
	   server_ent->h_length);

    soc = socket(AF_INET, SOCK_STREAM, 0);
    if (soc < 0) {
#if defined(WIN32)
	fprintf(stderr, "socket failed");
#else
	perror("socket");
#endif
	return -1;
    }

    if (connect(soc, (struct sockaddr *)&server, sizeof(server)) == -1) {

#if defined(WIN32)
	fprintf(stderr, "connect failed");
#else
	perror("connect");
#endif
	return -1;
    }
    fprintf(stderr, "connected.\n");
    return soc;
}

char *
chop_newline(char *buf, int len)
{
    int n = strlen(buf);

    if (n < len && buf[n - 1] == '\n') {
	buf[n - 1] = '\0';
    }
    return buf;
}

void
print_buf(char *buf, int nbyte)
{
    int i, j;

    printf("\n");
    for (j = 0; j < nbyte / 4; j++) {
	for (i = 0; i < 4; i++) {
	    printf("%d ", buf[j * 4 + i]);
	}
	printf("   ");
    }
    printf("\n");
}

/*
 * cleanly exit the process.
 *
 * ret: exit code
 * n: number of fds to be closed.
 * fd: list of fds to be closed.
 */
int
die(int ret, int n, SOCKET *fd)
{
    int i;
    for (i = 0; i < n; i++) {
	closesocket(fd[i]);
    }
#if defined(WIN32)
    WSACleanup();
#endif
    exit(ret);
}

#if !defined(WIN32)
int
closesocket(SOCKET fd)
{
    return close(fd);
}
#endif

/*
 * OpenGL wrapper
 */

void
putString(char *s, double x, double y, int font)
{
    char buf[256];
    static int firstcall = 1;
    static void *fontlist[] = {
	GLUT_BITMAP_TIMES_ROMAN_24,
	GLUT_BITMAP_HELVETICA_18,
	GLUT_BITMAP_HELVETICA_12,
    };
    static int base[sizeof(fontlist)/sizeof(fontlist[0])];
    int nfont = sizeof(fontlist)/sizeof(fontlist[0]);

    if (firstcall) {
	int i, j;
	firstcall = 0;

	for (j = 0; j < nfont; j++) {
	    base[j] = glGenLists(128);
	    for (i = 0; i < 128; i++) {
		glNewList(base[j] + i, GL_COMPILE);
		// glutStrokeCharacter(GLUT_STROKE_MONO_ROMAN, i);
		glutBitmapCharacter(fontlist[j], i);
		glEndList();
	    }
	}
    }
    glListBase(base[font]);
    sprintf(buf, s);
    glRasterPos2d(x, y);
    glCallLists(strlen(buf), GL_BYTE, buf);
}

int
limit_range(int *val, int min, int max)
{
    int ret = 0;
    int v = *val;
    if (v < min) {
	v = min;
	ret = 1;
    }
    if (v > max) {
	v = max;
	ret = 1;
    }
    *val = v;
    return ret;
}

